module parsestring;
// version:0.01
/++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 + Coded by Larry Cowan
 + February 29, 2004
 + Re-released March 17, 2004
 +
 + --------- char[][] parseString ( char[] str, char[][] delim ) ----------
 +
 + This module takes a string and a string of delimiters as input and
 + returns an array of tokens (strings) which were found in the input.
 +
 + parseString() can be used to parse a path into dirname(s) and filename
 + by calling it with "\\/" as a delimiter list.  "drive" if given, will
 + be included in the bottom (first) directory reference.
 +
 + -------------- bit charInString ( char ch, char[] str ) ----------------
 +
 + An auxiliary function is provided which evaluates ( char in string ? )
 + and returns true or false.  This is case-sensitive.
 +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Copyright 2004 by Larry F. Cowan
 +  Distribution rights:  This code may be used for any purpose whatever,
 +      commercial or otherwise, without specific permission from the 
 +      author(s), as long as they are not misrepresented in any way, the
 +      author name(s) are kept in the source, and this notice is included.
 +      Changes should indicate at least an author name and date.  No listed
 +      author takes on any responsibility or liability for your use of this
 +      module. It's free.  Caveat emptor!  Any underlying (imported) modules
 +      are under their own licensing and this is not affected here. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++/
// debug=Parsestring;  // compile test main()
// debug=parsestring;  // compile debug messages

char[][] parseString ( char[] str, char[] delim )
in {
    assert( str.length > 0 );
    assert( delim.length > 0 );
}
out (result) {
    assert( result.length > 0 );
}
body {
  char[][] parse;
  int m = 0;
  int n = 0;

  parse.length = 4;
  parse[0].length = 8;
  
  debug (parsestring) printf("parseString( \"%.*s\", \"%.*s\" )\n",str,delim);

  foreach ( int ix, char ch; str )
  {
    if ( !charInString(ch,delim) )
    {
      if ( parse[n].length == m )
        parse[n].length = m + 8;
      parse[n][m] = ch;
      m++;
    }
    else
    {
      parse[n] = parse[n][0..m];
      debug (parsestring) printf("Parsed: %d\"%.*s\"\n",m,parse[n]);
      m = 0;
      n++;
      if ( parse.length == n )
        parse.length = n + 4;
      parse[n].length = 8;
    }
  }
  parse[n] = parse[n][0..m];
  debug (parsestring) printf("Parsed: %d\"%.*s\"\n",m,parse[n]);
  return parse[0..n+1];
}

bit charInString ( char c, char[] str )
in {
    assert( str.length > 0 );
}
body {
    foreach ( char ch; str )
      if ( c == ch )
        return true;
    return false;
}

/* ************************************** *
 *     main() - used for testing only     *
 * ************************************** */

debug (Parsestring) {

  void main ( )
  {
    char[][] tbl;
    char[] path1 = ".\\one\\two\\three_hundred_thirty_three\\fourth level\\file";
    char[] path2 = "one/two/three_hundred_thirty_three/fourth level/file";
    char[] path3 = "/one\\two/three_hundred_thirty_three\\fourth level/file";
  
    printf("\n%.*s\n",path1);
    tbl = parseString(path1,"\\");
    foreach ( int ix, char[] str; tbl )
      printf("%-8.*s: %.*s\n"
            , (ix == tbl.length - 1) ? "FILE" : "DIR"
            , str );
    printf("\n%.*s\n",path2);
    tbl = parseString(path2,"/");
    foreach ( int ix, char[] str; tbl )
      printf("%-8.*s: %.*s\n"
            , (ix == tbl.length - 1) ? "FILE" : "DIR"
            , str );
    printf("\n%.*s\n",path3);
    tbl = parseString(path3,"\\/");
    foreach ( int ix, char[] str; tbl )
      printf("%-8.*s: %.*s\n"
            , (ix == tbl.length - 1) ? "FILE" : "DIR"
            , str );
  }
}

